#property copyright "Copyright 2009 by asm8086."
#property link      "http://www.forexfactory.com/showpost.php?p=2330567&postcount=147"

#property indicator_buffers 1
#property indicator_chart_window
#property indicator_color1 White
#property indicator_width1 2

extern string CountHelp = "Number of bars to process";
extern int Count = 256;
extern string TypeHelp = "Valid values are : 0, 1, 2 or 3";
extern int Type = 1;
extern string PriceHelp = "Close 0, Open 1, High 2, Low 3, Median 4, Typical 5, Weighted 6";
extern int Price = PRICE_MEDIAN;

#define Length 39
double PI = 3.1415926535897932384626433832795;
double Coefficients[Length];
double Buffer[];

int init()
{
	IndicatorBuffers(1);
	IndicatorDigits(6);
	SetIndexBuffer(0, Buffer);
	ArraySetAsSeries(Buffer, true);
	IndicatorShortName(StringConcatenate("Ultra Low Lag Moving Average, Type ", Type));

	Design();
	return(0);
}

int start()
{
	if(Bars < Length + Count)
	{
		return(-1);
	}

	ArrayInitialize(Buffer,EMPTY_VALUE);
	DirectFormFIR();
	return(0);
}

void DirectFormFIR()
{
	for(int i = 0; i < Count; i++)
	{
		double sum = 0.0;

		for(int n = 0; n < Length; n++)
		{
			sum += iMA(NULL, 0, 1, 0, 0, Price, i + n) * Coefficients[n];
		}

		Buffer[i] = sum;
	}
}

void Design()
{
	double Cycles;
	
	switch(MathAbs(Type))
	{
		case 0:
			Cycles = 1.25;
			break;
		case 1:
			Cycles = 2.5;
			break;
		case 2:
			Cycles = 3.75;
			break;
		case 3:
			Cycles = 6.25;
			break;
		default:
			Cycles = 6.25;
			break;
	}
	
	double sum = 0.0;
	double w = (double)Length;
	
	for(int n = 0; n < Length; n++)
	{
		double x = Sinc(((double)n / w) * (Cycles * PI));
		Coefficients[n] = x;
		sum += x;
	}
	
	sum = 1.0 / sum;
	
	for(int i = 0; i < Length; i++)
	{
		Coefficients[i] *= sum;
	}
}

double Sinc(double x)
{
	if(x == 0.0)
	{
		return(1.0);
	}
	else
	{
		return(MathSin(PI * x) / (PI * x));
	}
}

